/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package scalapi.jdk

import control.{Use, Evaluate}
import java.util.zip.{ZipEntry, ZipInputStream}
import scalapi.jdk.io.ZipInputStreams
import scala.concurrent.Future
import scalapi.jdk.concurrent.Futures.futureToValue
import scala.concurrent.duration._
import java.io.InputStream

/**
 * Global import point of Scalapi JDK implicit conversions.
 */
object Implicits {

  // Any

  implicit class RichAny[T](any: T) {

    def assertNotNull: T =
      Assert.assertNotNull(any)

  }

  // Concurrent

  implicit class RichFuture[T](future: Future[T]) {

    def getValue(implicit atMost: Duration = 5.seconds): T =
      futureToValue(future)(atMost)

  }

  // Control

  implicit def pimpAnyWithEvaluate[A](any: A) = new Evaluate.AnyPimp(any)

  implicit def pimpClosableWithUse[C <: {def close()}](closable: C) = new Use.ClosablePimp(closable)

  // IO

  implicit class RichInputStream(inputStream: InputStream) {

    def toZipInputStream =
      new ZipInputStream(inputStream)

  }

  implicit class RichZipInputStream(zipInputStream: ZipInputStream) {

    def mapEntries[T](entryHandler: ZipEntry => T): Seq[T] =
      ZipInputStreams.mapEntries(zipInputStream)(entryHandler)

    def forEachEntry(entryHandler: ZipEntry => Unit) {
      ZipInputStreams.forEachEntry(zipInputStream)(entryHandler)
    }

  }

}